﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace CalcHelper
{
    public partial class GoToPage : Form
    {
        public uint selPage {get; set;}
        public uint totalPages;

        public GoToPage(uint numPages)
        {
            InitializeComponent();

            totalPages = numPages;

            CancelButton = canGoPageBtn;
            AcceptButton = okGoPageBtn;

            goPageLabel.Text = string.Format("Page Range (1 to {0})", numPages);
        }

        private void okGoPageBtn_Click(object sender, EventArgs e)
        {
            uint page = 0;

            uint.TryParse(goPageTextBox.Text, out page);

            if ((page >= 1) && (page <= totalPages))
            {
                this.selPage = page;
                this.DialogResult = DialogResult.OK;
                this.Close();
            }
            else
            {
                CustomToolTip toolTip = new CustomToolTip(goPageTextBox, "  Not In Range", "W");
                toolTip.ShowToolTip(string.Format("{0}The page you selected is out of range.{0} ", Environment.NewLine));
            }
        }

        private void canGoPageBtn_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Cancel;
            this.Close();
        }

        private void GoPageTextBox_KeyPress(object sender, KeyPressEventArgs e)
        {
            // Allow backspace key.
            if (e.KeyChar != '\b')
            {
                /*
                 * Allow just digits 0-9.
                 * "Set Handled to true to cancel the KeyPress event",
                 * so a 'bad' char sets e.handled to true which 'ignores'
                 * the char and we inform user.
                */
                e.Handled = !char.IsDigit(e.KeyChar);

                if (e.Handled)
                {
                    CustomToolTip toolTip = new CustomToolTip(goPageTextBox, "  Unacceptable Character", "E");
                    toolTip.ShowToolTip(string.Format("{0}You can only type a number here.{0} ", Environment.NewLine));
                }
            }
        }

        private void GoToPage_Shown(object sender, EventArgs e)
        {
            goPageTextBox.Select();
        }
    }

    /*
     * Obtained from Code Project, Author: Nishith Jain [India].
     * I tweaked it a bit.
    */
    class CustomToolTip : ToolTip
    {
        Control textBox;

        public CustomToolTip(Control ctrl, string title, string icon)
        {

            textBox = ctrl;

            /*
             * Need to set these to false so we get the rectangle tooltip box to
             * work correctly. Not going the IsBalloon = true route because the
             * 'tip arrow' sometimes emerges from the top of the balloon [correct]
             * and sometimes from the bottom.
            */ 
            this.OwnerDraw = false;
            this.IsBalloon = false;

            this.ToolTipTitle = title;

            switch (icon)
            {
                case ("E"):
                    this.ToolTipIcon = System.Windows.Forms.ToolTipIcon.Error;
                    break;

                case ("W"):
                    this.ToolTipIcon = System.Windows.Forms.ToolTipIcon.Warning;
                    break;

                default:
                    this.ToolTipIcon = System.Windows.Forms.ToolTipIcon.Info;
                    break;
            }
        }

        public void ShowToolTip(string msg)
        {
            this.UseAnimation = true;

            Graphics g = textBox.CreateGraphics();

            // Get the exact width of the string
            int width = (int)g.MeasureString(textBox.Text, textBox.Font).Width;

            g.Dispose();

            // There is a bug in the tool tip. So display an empty string
            // for 0ms before displaying the actual string for 4 seconds. 
            this.Show(string.Empty, textBox, 0);
            this.Show(msg, textBox, width, textBox.Height, 4000);
        }
    }
}
